package com.bwie.home.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.domain.*;
import com.bwie.common.domain.request.HouseEsRequest;
import com.bwie.common.domain.request.VoIndexList;
import com.bwie.home.mapper.HouseMapper;
import com.bwie.home.service.*;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

@Service
public class HouseServiceImpl extends ServiceImpl<HouseMapper, House>
        implements HouseService {

    private final HouseTypeService houseTypeService;
    private final DictionaryTypeService dictionaryTypeService;
    private final HouseImagesService houseImagesService;
    private final UserService userService;
    private final AdviceService adviceService;
    private final Executor executor;


    public HouseServiceImpl(HouseTypeService houseTypeService,
                            DictionaryTypeService dictionaryTypeService,
                            HouseImagesService houseImagesService,
                            UserService userService,
                            AdviceService adviceService, Executor executor) {
        this.houseTypeService = houseTypeService;
        this.dictionaryTypeService = dictionaryTypeService;
        this.houseImagesService = houseImagesService;
        this.userService = userService;
        this.adviceService = adviceService;
        this.executor = executor;
    }

    @Override
    public List<HouseEsRequest> getHouseEsList() {

        List<House> list = this.list();
        List<HouseEsRequest> esRequestList = list.stream().map(c -> {
            HouseType houseType = houseTypeService.getById(c.getHouseTypeId());
            DictionaryType dictionaryType = dictionaryTypeService.getById(c.getDictionaryTypeId());
            List<HouseImages> houseImagesList = houseImagesService.list(new LambdaQueryWrapper<HouseImages>()
                    .eq(HouseImages::getHouseId, c.getId()));
            List<String> collect = houseImagesList.stream().map(img -> {
                return img.getImageAddr();
            }).collect(Collectors.toList());
            HouseEsRequest houseEsRequest = new HouseEsRequest();
            BeanUtils.copyProperties(c, houseEsRequest);
            houseEsRequest.setHouseTypeName(houseType.getName());
            houseEsRequest.setDictionaryTypeName(dictionaryType.getDictionaryTypeLabel());
            houseEsRequest.setHouseImages(collect);
            return houseEsRequest;
        }).collect(Collectors.toList());

        return esRequestList;
    }

    @Override
    public VoIndexList getIndexList() {

        CountDownLatch countDownLatch = new CountDownLatch(5);
        VoIndexList voIndexList = new VoIndexList();

        //横幅信息
        CompletableFuture<List<HouseImages>> f1 = CompletableFuture.supplyAsync(() -> {
            List<HouseImages> list = houseImagesService.list(new LambdaQueryWrapper<HouseImages>().eq(HouseImages::getHouseId, 0));
            countDownLatch.countDown();
            return list;
        }, executor);

        // 经纪人信息
        CompletableFuture<List<User>> f2 = CompletableFuture.supplyAsync(() -> {
            List<User> list = userService.list();
            countDownLatch.countDown();
            return list;
        }, executor);

        //咨询
        CompletableFuture<List<Advice>> f3 = CompletableFuture.supplyAsync(() -> {
            List<Advice> list = adviceService.list();
            countDownLatch.countDown();
            return list;
        }, executor);

        //新房
        CompletableFuture<List<House>> f4 = CompletableFuture.supplyAsync(() -> {
            List<House> list = this.list(new LambdaQueryWrapper<House>().last("LIMIT" + 5));
            countDownLatch.countDown();
            return list;
        }, executor);

        //二手房
        CompletableFuture<List<House>> f5 = CompletableFuture.supplyAsync(() -> {
            List<House> list = this.list(new LambdaQueryWrapper<House>().last("LIMIT" + 5));
            countDownLatch.countDown();
            return list;
        }, executor);

        try {
            countDownLatch.await();
            List<HouseImages> houseImages = f1.get();
            List<String> imageList = houseImages.stream().map(HouseImages::getImageAddr).collect(Collectors.toList());
            voIndexList.setHouseImage(imageList);//横幅信息
            voIndexList.setUserInfo(f2.get());
            voIndexList.setAdviceList(f3.get());
//            voIndexList.setNewHouse(f4.get());
//            voIndexList.setOldHouse(f5.get());

        } catch (Exception e) {
            throw new RuntimeException(e);
        }


        return voIndexList;
    }
}
